﻿#MoxPose2Shelf for python
#20180215 TRSだけ保存するモード搭載
import maya.cmds as cmds
import maya.mel
import os
import random
import datetime

gShelfTopLevel = maya.mel.eval('$tmpVar=$gShelfTopLevel')
scriptPath = cmds.internalVar(uad=True)+"scripts"
iconPath = scriptPath+"/MoxPose2Shelf_icons/"

#現在のカレントフレームを取得して３桁の文字列に並べ替えます。
def currentTimePadding():
    strCurrentTime = str(int(cmds.currentTime(q=True))).zfill(3)
    return strCurrentTime


#現在のフレームのスクリーンショットを撮影します。
def createIcon(label):
    tfilename = cmds.file(q=True,shn=True,sn=True)
    fname, ext = os.path.splitext(tfilename)

    if len(fname) == 0:
        fname="untitled"
    
    name = fname + "_" + label + "_" + datetime.datetime.today().strftime("%d%H%M")
    iTime = int(cmds.currentTime(q=True))#現在のフレームを取得
    sTime = currentTimePadding()#3桁の数字に並び替える
    
    #パスがなければパスを作る
    if os.path.isdir(iconPath):pass
    else:os.mkdir(iconPath)
    
    cmds.modelEditor ("modelPanel4",e=True, nurbsCurves=False)#ナーブスカーブ非表示
    
    #スクショを取る
    cmds.displayPref(displayAffected=0)
    cmds.playblast(format="image", filename =iconPath+"/"+name,st=iTime,et=iTime,
        forceOverwrite=True, sequenceTime=0, clearCache =1, 
        viewer =0, showOrnaments= 0, fp=3 ,percent=100 , 
        compression ="jpg" ,quality=70,w=32,h=32)
        
    posejpg = iconPath + "/" + name+ "." + sTime + ".jpg"
    
    if os.path.isfile(iconPath + "/" + name + "." + sTime + ".jpeg"):#maya2012のバグ対策用 jpegになるので,jpgに変換する
        if os.path.isfile(posejpg):os.remove(posejpg)
        os.rename(iconPath + "/" + name+ "." + sTime + ".jpeg", posejpg)
    
    if cmds.modelEditor ("modelPanel4",ex=True, nurbsCurves=False):cmds.modelEditor ("modelPanel4",e=True, nurbsCurves=True)#ナーブスカーブ表示
    return posejpg
    

#ポーズを格納する関数
def saveSelectedNodeAttrsToShelf(label):
    selected = cmds.ls(sl=True)
    if len(selected)== 0:
        cmds.warning (u"何も選択されていません。")
        return "Nothing is currently selected. No shelf button created, or action taken."
        
    #ソース生成
    attrDict = "dic={"
    for node in selected:
        if cmds.checkBox("OnlyTRS", q=1, v=1) == 1:
            keyableAttrs = cmds.listAttr (node,r=1,w=1, k=1, u=1, v=1,st=['translate*','rotate*','scale*'])
        else:
            keyableAttrs = cmds.listAttr (node, r=True, w=True, k=True, u=True, v=True, m=True, s=True)
        if keyableAttrs is None:
            continue
        elif len(keyableAttrs) == 0:
            continue  
         
        for attr in keyableAttrs:
            value = cmds.getAttr(node+"."+attr)
            attrDict = attrDict + ("\"" +node +"."+attr+"\": "+str(value)+",\n")
    attrDict = attrDict + "}\n"
  
    starcode = """
import maya.cmds as cmds
trslist = ["translateX","translateY","translateZ","rotateX","rotateY","rotateZ"]
sellist = cmds.ls(sl=True)
if len(sellist) == 0:
    for k,v in dic.items():cmds.setAttr(k, v) 
for rigCtrl in sellist:
    for trs in trslist:
        try:cmds.setAttr(rigCtrl+"."+trs, dic[rigCtrl+"."+trs])
        except:pass""" 
    
    starcode2 = """
import maya.cmds as cmds

def NSE(mode):
    newdic = {}
    bef = cmds.textField("replace1",q=1, text=1)
    aft = cmds.textField("replace2",q=1, text=1)
    add = cmds.textFieldButtonGrp("add",q=1, text=1)
    trslist = ["translateX","translateY","translateZ","rotateX","rotateY","rotateZ"]
    sellist = cmds.ls(sl=True)
    if mode=="replace":
        for k,v in dic.items():newdic[k.replace(bef,aft)] = v              
    else:
        for k,v in dic.items(): newdic[add+k] = v

    if len(sellist) == 0:
        for k,v in newdic.items():cmds.setAttr(k, v)    
    else:   
        for rigCtrl in sellist:
            for trs in trslist:
                if rigCtrl+"."+trs in newdic:
                    cmds.setAttr(rigCtrl+"."+trs, newdic[rigCtrl+"."+trs])
                else:pass
    cmds.deleteUI( "NameSpaceEdit" )
       

class NameSpaceEdit(object):
    def __init__(self):
        if cmds.window("NameSpaceEdit",ex=True):cmds.deleteUI( "NameSpaceEdit" )
        cmds.window ("NameSpaceEdit", w=200, sizeable=1, tlb=True)
        cmds.columnLayout()
        cmds.rowLayout(nc=5);cmds.text("repl");cmds.textField("replace1", w=54);cmds.text(">")
        cmds.textField("replace2", w=54);cmds.button("rep",l="done", c="NSE(mode='replace')");cmds.setParent('..')
        cmds.textFieldButtonGrp("add", l="add", buttonLabel="done",cw3=[20,120,20], bc="NSE(mode='add')")
        cmds.showWindow()
NameSpaceEdit = NameSpaceEdit()"""
 

    attrDict1 = attrDict + starcode
    attrDict2 = attrDict + starcode2
    
    #シェルフボタン生成       
    global gShelfTopLevel
    if cmds.tabLayout( gShelfTopLevel, exists=True):
        if (label == ""):label = "pose"
        posejpg = createIcon(label)
        if cmds.checkBox("labelVisibility", q=True, v=True):pass
        else:label=""
        if cmds.checkBox("screenShot", q=True, v=True)  :pass
        else: posejpg = "menuIconCharacters.xpm"
        cmds.shelfButton(
            parent =gShelfTopLevel + "|" + cmds.tabLayout(gShelfTopLevel,q=True, st=True),
            enableCommandRepeat=1,
            enable= 1,
            sourceType = "python",
            width = 34,
            height = 34,
            manage = 1,
            visible = 1,
            annotation = label,
            label = label,
            iol = label,
            olb = (0,0,0, 0.2),
            image1 = posejpg,
            style = "iconOnly", 
            command = attrDict1,
            doubleClickCommand = attrDict2)
    else:
        cmds.error(u"シェルフボタンを追加するシェルフタブを表示してください。")


#選択リストを格納する関数
def saveNodeSelectionToShelf(label):
    selected = cmds.ls(sl=True)
    if len(selected) == 0:
        cmds.warning (u"何も選択されていません。")
        return "Nothing is currently selected. No shelf button created, or action taken." 
        
    #ソース生成
    attrDict = ""
    for node in selected:
        attrDict = "cmds.select(\""+node+"\", add=True)\n"+attrDict
    attrDict = "cmds.select( cl=True )\n"+attrDict
    
    #シェルフボタン生成   
    global gShelfTopLevel
    if cmds.tabLayout (gShelfTopLevel,exists=True):
        if label == "":
            label = "select"
        cmds.shelfButton(
            parent = gShelfTopLevel + "|" + cmds.tabLayout(gShelfTopLevel, q=True, st=True),
            enableCommandRepeat= 1,
            enable = 1,
            sourceType = "python",
            width = 34,
            height = 34,
            manage = 1,
            visible = 1,
            annotation = label,
            label = label ,
            iol = label,
            image1 = "aselect.xpm" ,
            style = "iconOnly" ,
            command = attrDict)
    else:
        cmds.error (u"シェルフボタンを追加するシェルフタブを表示してください。")

#UI
class MP2Smain(object):
    def __init__(self):
        if cmds.window("MoxPose2Shelf",ex=True):cmds.deleteUI( "MoxPose2Shelf" )
 
        cmds.window ("MoxPose2Shelf", w =140, sizeable=0, tlb=True)
        cmds.columnLayout() 
        
        cmds.checkBox("screenShot", l="screenShot", v=True)
        cmds.checkBox("labelVisibility", l="label Visibility", v=0)
        cmds.checkBox("OnlyTRS", l="Only TRS Mode", v=1)
        cmds.textField ("Label",w = 100)

        cmds.button (h=35, w=100,label ="pose",c="MoxPose2Shelf.saveSelectedNodeAttrsToShelf( cmds.textField('Label', q=True, tx=True ))" )
        cmds.text ("Or", l="")
        cmds.button (w= 100,label= "selection",c ="MoxPose2Shelf.saveNodeSelectionToShelf( cmds.textField('Label', q=True, tx=True))")

        cmds.showWindow()
        
MP2SmainWind = MP2Smain()    
    